@@ -24,6 +24,7 @@
 
 #include "gnunet_chat_handle.h"
 
+#include "gnunet_chat_account.h"
 #include "gnunet_chat_handle_intern.c"
 #include "gnunet_chat_message.h"
 #include <gnunet/gnunet_arm_service.h>
@@ -58,37 +59,31 @@ handle_create_from_config (const struct GNUNET_CONFIGURATION_Handle* cfg,
 
   handle->directory = NULL;
 
+  char *dir_path = NULL;
   if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_filename(cfg,
 		     GNUNET_MESSENGER_SERVICE_NAME,
 		     "MESSENGER_DIR",
-		     &(handle->directory)))
+		     &dir_path))
   {
-    if (handle->directory)
-      GNUNET_free(handle->directory);
-
-    handle->directory = NULL;
+    if (dir_path)
+      GNUNET_free(dir_path);
   }
-  else if ((GNUNET_YES != GNUNET_DISK_directory_test(handle->directory, GNUNET_YES)) &&
-	         (GNUNET_OK != GNUNET_DISK_directory_create(handle->directory)))
+  else if ((GNUNET_YES != GNUNET_DISK_directory_test(dir_path, GNUNET_YES)) &&
+	         (GNUNET_OK != GNUNET_DISK_directory_create(dir_path)))
   {
-    GNUNET_free(handle->directory);
-
-    handle->directory = NULL;
+    GNUNET_free(dir_path);
   }
-
-  if (handle->directory)
+  else
   {
     char *chat_directory = NULL;
-    util_get_dirname(handle->directory, "chat", &chat_directory);
+    util_get_dirname(dir_path, "chat", &chat_directory);
+    GNUNET_free(dir_path);
 
     if ((GNUNET_YES != GNUNET_DISK_directory_test(chat_directory, GNUNET_YES)) &&
     	  (GNUNET_OK != GNUNET_DISK_directory_create(chat_directory)))
       GNUNET_free(chat_directory);
     else
-    {
-      GNUNET_free(handle->directory);
       handle->directory = chat_directory;
-    }
   }
 
   handle->msg_cb = msg_cb;
@@ -97,6 +92,8 @@ handle_create_from_config (const struct GNUNET_CONFIGURATION_Handle* cfg,
   handle->accounts_head = NULL;
   handle->accounts_tail = NULL;
 
+  handle->refreshing = GNUNET_NO;
+
   handle->next = NULL;
   handle->current = NULL;
   handle->monitor = NULL;
@@ -204,6 +201,9 @@ handle_destroy (struct GNUNET_CHAT_Handle *handle)
     handle->files, it_destroy_handle_files, NULL
   );
 
+  while (handle->attributes_head)
+    internal_attributes_destroy(handle->attributes_head);
+
   if (handle->reclaim)
     GNUNET_RECLAIM_disconnect(handle->reclaim);
 
@@ -326,7 +326,7 @@ handle_update_identity(struct GNUNET_CHAT_Handle *handle)
 
 void
 handle_connect (struct GNUNET_CHAT_Handle *handle,
-		            const struct GNUNET_CHAT_Account *account)
+		            struct GNUNET_CHAT_Account *account)
 {
   GNUNET_assert(
     (handle) && (account) &&
@@ -355,8 +355,10 @@ handle_connect (struct GNUNET_CHAT_Handle *handle,
   const struct GNUNET_CRYPTO_PrivateKey *key;
   key = account_get_key(account);
 
+  const char *name = account_get_name(account);
+
   handle->messenger = GNUNET_MESSENGER_connect(
-    handle->cfg, account->name, key,
+    handle->cfg, name, key,
     on_handle_message,
     handle
   );
@@ -497,19 +499,26 @@ handle_disconnect (struct GNUNET_CHAT_Handle *handle)
 }
 
 static struct GNUNET_CHAT_InternalAccounts*
-find_accounts_by_name (struct GNUNET_CHAT_Handle *handle,
-		                   const char *name)
+find_accounts_by_name (const struct GNUNET_CHAT_Handle *handle,
+		                   const char *name,
+                       enum GNUNET_GenericReturnValue skip_op)
 {
   GNUNET_assert((handle) && (name));
 
   struct GNUNET_CHAT_InternalAccounts *accounts = handle->accounts_head;
+  const char *account_name;
+
   while (accounts)
   {
-    if (!(accounts->account))
+    if ((!(accounts->account)) || ((GNUNET_YES == skip_op) &&
+        (accounts->op)))
       goto skip_account;
 
-    if ((accounts->account->name) &&
-        (0 == strcmp(accounts->account->name, name)))
+    account_name = account_get_name(
+      accounts->account
+    );
+
+    if ((account_name) && (0 == strcmp(account_name, name)))
       break;
 
   skip_account:
@@ -519,6 +528,22 @@ find_accounts_by_name (struct GNUNET_CHAT_Handle *handle,
   return accounts;
 }
 
+struct GNUNET_CHAT_Account*
+handle_get_account_by_name (const struct GNUNET_CHAT_Handle *handle,
+		                        const char *name,
+                            enum GNUNET_GenericReturnValue skip_op)
+{
+  GNUNET_assert((handle) && (name));
+
+  struct GNUNET_CHAT_InternalAccounts *accounts;
+  accounts = find_accounts_by_name(handle, name, skip_op);
+
+  if (!accounts)
+    return NULL;
+
+  return accounts->account;
+}
+
 static enum GNUNET_GenericReturnValue
 update_accounts_operation (struct GNUNET_CHAT_InternalAccounts **out_accounts,
                            struct GNUNET_CHAT_Handle *handle,
@@ -553,7 +578,7 @@ handle_create_account (struct GNUNET_CHAT_Handle *handle,
   GNUNET_assert((handle) && (name));
 
   struct GNUNET_CHAT_InternalAccounts *accounts;
-  accounts = find_accounts_by_name(handle, name);
+  accounts = find_accounts_by_name(handle, name, GNUNET_NO);
 
   if (accounts)
     return GNUNET_SYSERR;
@@ -586,12 +611,18 @@ handle_create_account (struct GNUNET_CHAT_Handle *handle,
 
 enum GNUNET_GenericReturnValue
 handle_delete_account (struct GNUNET_CHAT_Handle *handle,
-		                   const char *name)
+		                   const struct GNUNET_CHAT_Account *account)
 {
-  GNUNET_assert((handle) && (name));
+  GNUNET_assert((handle) && (account));
 
   struct GNUNET_CHAT_InternalAccounts *accounts;
-  accounts = find_accounts_by_name(handle, name);
+  accounts = handle->accounts_head;
+
+  while (accounts)
+    if (account == accounts->account)
+      break;
+    else
+      accounts = accounts->next;
 
   if (!accounts)
     return GNUNET_SYSERR;
@@ -607,6 +638,8 @@ handle_delete_account (struct GNUNET_CHAT_Handle *handle,
   if (GNUNET_OK != result)
     return result;
 
+  const char *name = account_get_name(account);
+
   accounts->op = GNUNET_IDENTITY_delete(
     handle->identity,
     name,
@@ -622,17 +655,31 @@ handle_delete_account (struct GNUNET_CHAT_Handle *handle,
 
 enum GNUNET_GenericReturnValue
 handle_rename_account (struct GNUNET_CHAT_Handle *handle,
-                       const char *old_name,
+                       const struct GNUNET_CHAT_Account *account,
                        const char *new_name)
 {
-  GNUNET_assert((handle) && (old_name) && (new_name));
+  GNUNET_assert((handle) && (account) && (new_name));
 
   struct GNUNET_CHAT_InternalAccounts *accounts;
-  accounts = find_accounts_by_name(handle, old_name);
+  accounts = handle->accounts_head;
+
+  while (accounts)
+    if (account == accounts->account)
+      break;
+    else
+      accounts = accounts->next;
 
   if (!accounts)
     return GNUNET_SYSERR;
 
+  if (find_accounts_by_name(handle, new_name, GNUNET_NO))
+    return GNUNET_SYSERR;
+
+  const char *old_name = account_get_name(account);
+
+  if (0 == strcmp(old_name, new_name))
+    return GNUNET_OK;
+
   enum GNUNET_GenericReturnValue result;
   result = update_accounts_operation(
     &accounts, 
@@ -709,13 +756,7 @@ handle_get_directory (const struct GNUNET_CHAT_Handle *handle)
 {
   GNUNET_assert(handle);
 
-  if (!(handle->directory))
-    return NULL;
-
-  if (!(handle->current))
-    return handle->directory;
-  else
-    return handle->current->directory;
+  return handle->directory;
 }
 
 char*
@@ -742,17 +783,20 @@ handle_update (struct GNUNET_CHAT_Handle *handle)
 {
   GNUNET_assert((handle) && (handle->current));
 
-  const char *name = handle->current->name;
-
-  if (!name)
-    return GNUNET_SYSERR;
-
   struct GNUNET_CHAT_InternalAccounts *accounts;
-  accounts = find_accounts_by_name(handle, name);
+  accounts = handle->accounts_head;
+
+  while (accounts)
+    if (handle->current == accounts->account)
+      break;
+    else
+      accounts = accounts->next;
 
   if (!accounts)
     return GNUNET_SYSERR;
 
+  const char *name = account_get_name(handle->current);
+
   enum GNUNET_GenericReturnValue result;
   result = update_accounts_operation(
     &accounts, 
@@ -790,7 +834,7 @@ handle_get_key (const struct GNUNET_CHAT_Handle *handle)
 
 void
 handle_send_internal_message (struct GNUNET_CHAT_Handle *handle,
-                              const struct GNUNET_CHAT_Account *account,
+                              struct GNUNET_CHAT_Account *account,
                               struct GNUNET_CHAT_Context *context,
                               enum GNUNET_CHAT_MessageFlag flag,
                               const char *warning,
