@@ -25,6 +25,16 @@
 #ifndef GNUNET_CHAT_HANDLE_H_
 #define GNUNET_CHAT_HANDLE_H_
 
+#include "gnunet_chat_lib.h"
+#include "gnunet_chat_account.h"
+#include "gnunet_chat_lobby.h"
+#include "gnunet_chat_message.h"
+#include "gnunet_chat_uri.h"
+
+#include "internal/gnunet_chat_accounts.h"
+#include "internal/gnunet_chat_attribute_process.h"
+#include "internal/gnunet_chat_ticket_process.h"
+
 #include <gnunet/gnunet_common.h>
 #include <gnunet/gnunet_arm_service.h>
 #include <gnunet/gnunet_fs_service.h>
@@ -37,16 +47,6 @@
 #include <gnunet/gnunet_time_lib.h>
 #include <gnunet/gnunet_util_lib.h>
 
-#include "gnunet_chat_lib.h"
-#include "gnunet_chat_account.h"
-#include "gnunet_chat_lobby.h"
-#include "gnunet_chat_message.h"
-#include "gnunet_chat_uri.h"
-
-#include "internal/gnunet_chat_accounts.h"
-#include "internal/gnunet_chat_attribute_process.h"
-#include "internal/gnunet_chat_ticket_process.h"
-
 struct GNUNET_CHAT_Handle;
 
 struct GNUNET_CHAT_InternalServices
@@ -106,8 +106,10 @@ struct GNUNET_CHAT_Handle
   struct GNUNET_CHAT_InternalAccounts *accounts_head;
   struct GNUNET_CHAT_InternalAccounts *accounts_tail;
 
-  const struct GNUNET_CHAT_Account *next;
-  const struct GNUNET_CHAT_Account *current;
+  enum GNUNET_GenericReturnValue refreshing;
+
+  struct GNUNET_CHAT_Account *next;
+  struct GNUNET_CHAT_Account *current;
   struct GNUNET_NAMESTORE_ZoneMonitor *monitor;
 
   struct GNUNET_CHAT_InternalLobbies *lobbies_head;
@@ -180,7 +182,7 @@ handle_destroy (struct GNUNET_CHAT_Handle *handle);
  */
 void
 handle_connect (struct GNUNET_CHAT_Handle *handle,
-		            const struct GNUNET_CHAT_Account *account);
+		            struct GNUNET_CHAT_Account *account);
 
 /**
  * Disconnects a given chat <i>handle</i> from its current
@@ -191,6 +193,20 @@ handle_connect (struct GNUNET_CHAT_Handle *handle,
 void
 handle_disconnect (struct GNUNET_CHAT_Handle *handle);
 
+/**
+ * Searches for an existing chat account by <i>name</i> as
+ * identifier for a given chat <i>handle</i>.
+ *
+ * @param[in] handle Chat handle
+ * @param[in] name Chat account name
+ * @param[in] skip_op Whether to skip accounts with active operation
+ * @return Chat account
+ */
+struct GNUNET_CHAT_Account*
+handle_get_account_by_name (const struct GNUNET_CHAT_Handle *handle,
+		                        const char *name,
+                            enum GNUNET_GenericReturnValue skip_op);
+
 /**
  * Enqueues a creation for a chat account with a specific
  * <i>name</i> as identifier for a given chat <i>handle</i>.
@@ -204,30 +220,29 @@ handle_create_account (struct GNUNET_CHAT_Handle *handle,
 		                   const char *name);
 
 /**
- * Enqueues a deletion for a chat account with a specific
- * <i>name</i> as identifier for a given chat <i>handle</i>.
+ * Enqueues a deletion for a chat <i>account</i> of a 
+ * given chat <i>handle</i>.
  *
  * @param[in,out] handle Chat handle
- * @param[in] name Chat account name
+ * @param[in] account Chat account
  * @return #GNUNET_OK on success, otherwise #GNUNET_SYSERR
  */
 enum GNUNET_GenericReturnValue
 handle_delete_account (struct GNUNET_CHAT_Handle *handle,
-		                   const char *name);
+		                   const struct GNUNET_CHAT_Account *account);
 
 /**
- * Renames a chat account with a specific <i>old_name</i>
- * as identifier for a given chat <i>handle</i> to another
- * specific <i>new_name</i>.
+ * Renames a chat <i>account</i> of a given chat 
+ * <i>handle</i> to another specific <i>new_name</i>.
  *
  * @param[in,out] handle Chat handle
- * @param[in] old_name Old chat account name
+ * @param[in] account Chat account
  * @param[in] new_name New chat account name
  * @return #GNUNET_OK on success, otherwise #GNUNET_SYSERR
  */
 enum GNUNET_GenericReturnValue
 handle_rename_account (struct GNUNET_CHAT_Handle *handle,
-                       const char *old_name,
+                       const struct GNUNET_CHAT_Account *account,
                        const char *new_name);
 
 /**
@@ -297,7 +312,7 @@ handle_get_key (const struct GNUNET_CHAT_Handle *handle);
  * as instant feedback.
  *
  * @param[in,out] handle Chat handle
- * @param[in] account Chat account or NULL
+ * @param[in,out] account Chat account or NULL
  * @param[in,out] context Chat context or NULL
  * @param[in] flag Chat message flag
  * @param[in] warning Warning text
@@ -305,7 +320,7 @@ handle_get_key (const struct GNUNET_CHAT_Handle *handle);
  */
 void
 handle_send_internal_message (struct GNUNET_CHAT_Handle *handle,
-                              const struct GNUNET_CHAT_Account *account,
+                              struct GNUNET_CHAT_Account *account,
                               struct GNUNET_CHAT_Context *context,
                               enum GNUNET_CHAT_MessageFlag flag,
                               const char *warning,
