@@ -45,6 +45,7 @@
 #include <gnunet/gnunet_scheduler_lib.h>
 #include <gnunet/gnunet_time_lib.h>
 #include <gnunet/gnunet_util_lib.h>
+
 #include <stdio.h>
 #include <string.h>
 
@@ -311,7 +312,10 @@ on_handle_gnunet_identity (void *cls,
   struct GNUNET_CHAT_Handle* handle = cls;
 
   if ((!ctx) || (!ego))
+  {
+    handle->refreshing = GNUNET_YES;
     goto send_refresh;
+  }
 
   struct GNUNET_CHAT_InternalAccounts *accounts = handle->accounts_head;
 
@@ -370,15 +374,13 @@ skip_account:
     account_create_from_ego(ego, name)
   );
 
-  if (handle->directory)
-    account_update_directory(accounts->account, handle->directory);
-
 send_refresh:
-  if (handle->refresh)
+  if ((GNUNET_YES != handle->refreshing) ||
+      (handle->refresh))
     return;
   
   handle->refresh = GNUNET_SCHEDULER_add_with_priority(
-    GNUNET_SCHEDULER_PRIORITY_BACKGROUND,
+    GNUNET_SCHEDULER_PRIORITY_IDLE,
     on_handle_refresh,
     handle
   );
@@ -927,17 +929,20 @@ skip_msg_handing:
     }
     case GNUNET_MESSENGER_KIND_SUBSCRIBE:
     {
-      const struct GNUNET_ShortHashCode *id = &(message->msg->body.subscribe.discourse);
+      const struct GNUNET_ShortHashCode *sid = &(message->msg->body.subscribe.discourse);
       struct GNUNET_CHAT_Discourse *discourse = GNUNET_CONTAINER_multishortmap_get(
-        context->discourses, id
+        context->discourses, sid
       );
 
       if (! discourse)
       {
-        discourse = discourse_create(context, id);
+        struct GNUNET_CHAT_DiscourseId id;
+        util_discourse_id_from_shorthash(sid, &id);
+
+        discourse = discourse_create(context, &id);
 
         if (GNUNET_OK != GNUNET_CONTAINER_multishortmap_put(context->discourses,
-          id, discourse, GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
+          sid, discourse, GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_FAST))
         {
           discourse_destroy(discourse);
           break;
